@@ -20,7 +20,7 @@
  *  along with this program ; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * $Id$
+ * $Id: bitstream.c,v 1.55.2.1 2006/07/10 15:05:30 Isibaar Exp $
  *
  ****************************************************************************/
 
@@ -33,19 +33,26 @@
 #include "mbcoding.h"
 
 
-static uint32_t __inline
-log2bin(uint32_t value)
+static const uint8_t log2_tab_16[16] =  { 0, 1, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4, 4, 4, 4, 4 };
+ 
+static uint32_t __inline log2bin(uint32_t value)
 {
-	int n = 0;
-
-	while (value) {
-		value >>= 1;
-		n++;
-	}
-	return n;
+  int n = 0;
+  if (value & 0xffff0000) {
+    value >>= 16;
+    n += 16;
+  }
+  if (value & 0xff00) {
+    value >>= 8;
+    n += 8;
+  }
+  if (value & 0xf0) {
+    value >>= 4;
+    n += 4;
+  }
+ return n + log2_tab_16[value];
 }
 
-
 static const uint32_t intra_dc_threshold_table[] = {
 	32,							/* never use */
 	13,
@@ -58,7 +65,7 @@ static const uint32_t intra_dc_threshold_table[] = {
 };
 
 
-void
+static void
 bs_get_matrix(Bitstream * bs,
 			  uint8_t * matrix)
 {
@@ -187,7 +194,6 @@ read_video_packet_header(Bitstream *bs,
 				DPRINTF(XVID_DEBUG_HEADER,"fcode_backward %i\n", *fcode_backward);
 			}
 		}
-
 	}
 
 	if (dec->newpred_enable)
@@ -381,7 +387,6 @@ int
 BitstreamReadHeaders(Bitstream * bs,
 					 DECODER * dec,
 					 uint32_t * rounding,
-					 uint32_t * reduced_resolution,
 					 uint32_t * quant,
 					 uint32_t * fcode_forward,
 					 uint32_t * fcode_backward,
@@ -418,18 +423,16 @@ BitstreamReadHeaders(Bitstream * bs,
 			DPRINTF(XVID_DEBUG_STARTCODE, "</visual_object_sequence>\n");
 
 		} else if (start_code == VISOBJ_START_CODE) {
-			int visobj_ver_id;
-
 			DPRINTF(XVID_DEBUG_STARTCODE, "<visual_object>\n");
 
 			BitstreamSkip(bs, 32);	/* visual_object_start_code */
 			if (BitstreamGetBit(bs))	/* is_visual_object_identified */
 			{
-				visobj_ver_id = BitstreamGetBits(bs, 4);	/* visual_object_ver_id */
-				DPRINTF(XVID_DEBUG_HEADER,"visobj_ver_id %i\n", visobj_ver_id);
+				dec->ver_id = BitstreamGetBits(bs, 4);	/* visual_object_ver_id */
+				DPRINTF(XVID_DEBUG_HEADER,"visobj_ver_id %i\n", dec->ver_id);
 				BitstreamSkip(bs, 3);	/* visual_object_priority */
 			} else {
-				visobj_ver_id = 1;
+				dec->ver_id = 1;
 			}
 
 			if (BitstreamShowBits(bs, 4) != VISOBJ_TYPE_VIDEO)	/* visual_object_type */
@@ -478,7 +481,7 @@ BitstreamReadHeaders(Bitstream * bs,
 				DPRINTF(XVID_DEBUG_HEADER,"ver_id %i\n", vol_ver_id);
 				BitstreamSkip(bs, 3);	/* video_object_layer_priority */
 			} else {
-				vol_ver_id = 1;
+				vol_ver_id = dec->ver_id;
 			}
 
 			dec->aspect_ratio = BitstreamGetBits(bs, 4);
@@ -544,16 +547,9 @@ BitstreamReadHeaders(Bitstream * bs,
 			dec->time_inc_resolution = BitstreamGetBits(bs, 16);	/* vop_time_increment_resolution */
 			DPRINTF(XVID_DEBUG_HEADER,"vop_time_increment_resolution %i\n", dec->time_inc_resolution);
 
-#if 0
-			dec->time_inc_resolution--;
-#endif
-
 			if (dec->time_inc_resolution > 0) {
 				dec->time_inc_bits = MAX(log2bin(dec->time_inc_resolution-1), 1);
 			} else {
-#if 0
-				dec->time_inc_bits = 0;
-#endif
 				/* for "old" xvid compatibility, set time_inc_bits = 1 */
 				dec->time_inc_bits = 1;
 			}
@@ -862,12 +858,8 @@ BitstreamReadHeaders(Bitstream * bs,
 				dec->shape == VIDOBJLAY_SHAPE_RECTANGULAR &&
 				(coding_type == P_VOP || coding_type == I_VOP)) {
 
-				*reduced_resolution = BitstreamGetBit(bs);
-				DPRINTF(XVID_DEBUG_HEADER, "reduced_resolution %i\n", *reduced_resolution);
-			}
-			else
-			{
-				*reduced_resolution = 0;
+				if (BitstreamGetBit(bs));
+					DPRINTF(XVID_DEBUG_ERROR, "RRV not supported (anymore)\n");
 			}
 
 			if (dec->shape != VIDOBJLAY_SHAPE_RECTANGULAR) {
@@ -992,6 +984,7 @@ BitstreamReadHeaders(Bitstream * bs,
 
 			BitstreamSkip(bs, 32);	/* user_data_start_code */
 
+			memset(tmp, 0, 256);
 			tmp[0] = BitstreamShowBits(bs, 8);
 
 			for(i = 1; i < 256; i++){
@@ -1081,16 +1074,11 @@ BitstreamWriteVolHeader(Bitstream * const bs,
 	int vol_profile = pParam->profile;
 
 	if ( (pParam->vol_flags & XVID_VOL_QUARTERPEL) ||
-         (pParam->vol_flags & XVID_VOL_GMC) ||
-		 (pParam->vol_flags & XVID_VOL_REDUCED_ENABLE))
+         (pParam->vol_flags & XVID_VOL_GMC))
 		vol_ver_id = 2;
 
-    if ((pParam->vol_flags & XVID_VOL_REDUCED_ENABLE)) {
-        vol_type_ind = VIDOBJLAY_TYPE_ART_SIMPLE;
-    }
-
-	if ((pParam->vol_flags & XVID_VOL_QUARTERPEL) ||
-        (pParam->vol_flags & XVID_VOL_GMC)) {
+    if ((pParam->vol_flags & (XVID_VOL_MPEGQUANT|XVID_VOL_QUARTERPEL|XVID_VOL_GMC|XVID_VOL_INTERLACING)) ||
+         pParam->max_bframes>0) {
         vol_type_ind = VIDOBJLAY_TYPE_ASP;
     }
 
@@ -1247,18 +1235,19 @@ BitstreamWriteVolHeader(Bitstream * const bs,
 
 	if (vol_ver_id != 1) {
 		BitstreamPutBit(bs, 0);		/* newpred_enable */
-		BitstreamPutBit(bs, (pParam->vol_flags & XVID_VOL_REDUCED_ENABLE)?1:0);
-									/* reduced_resolution_vop_enabled */
+		BitstreamPutBit(bs, 0);		/* reduced_resolution_vop_enabled */
 	}
 
 	BitstreamPutBit(bs, 0);		/* scalability */
 
 	BitstreamPadAlways(bs); /* next_start_code(); */
 
-	/* fake divx5 id, to ensure compatibility with divx5 decoder */
-#define DIVX5_ID "DivX999b000p"
-	if (pParam->max_bframes > 0 && (pParam->global_flags & XVID_GLOBAL_PACKED)) {
-		BitstreamWriteUserData(bs, DIVX5_ID, strlen(DIVX5_ID));
+	/* divx5 userdata string */
+#define DIVX5_ID ((char *)"DivX503b1393")
+  if ((pParam->global_flags & XVID_GLOBAL_DIVX5_USERDATA)) {
+    BitstreamWriteUserData(bs, DIVX5_ID, strlen(DIVX5_ID));
+  	if (pParam->max_bframes > 0 && (pParam->global_flags & XVID_GLOBAL_PACKED))
+      BitstreamPutBits(bs, 'p', 8);
 	}
 
 	/* xvid id */
@@ -1340,9 +1329,6 @@ BitstreamWriteVopHeader(
 	if ( (frame->coding_type == P_VOP) || (frame->coding_type == S_VOP) )
 		BitstreamPutBits(bs, frame->rounding_type, 1);
 
-	if ((frame->vol_flags & XVID_VOL_REDUCED_ENABLE))
-		BitstreamPutBit(bs, (frame->vop_flags & XVID_VOP_REDUCED)?1:0);
-
 	BitstreamPutBits(bs, 0, 3);	/* intra_dc_vlc_threshold */
 
 	if ((frame->vol_flags & XVID_VOL_INTERLACING)) {
@@ -1390,8 +1376,8 @@ BitstreamWriteVopHeader(
 
 void
 BitstreamWriteUserData(Bitstream * const bs,
-						uint8_t * data,
-						const int length)
+						const char *data,
+						const unsigned int length)
 {
 	int i;
 
@@ -1403,3 +1389,64 @@ BitstreamWriteUserData(Bitstream * const bs,
 	}
 
 }
+
+/*
+ * Group of VOP
+ */
+void
+BitstreamWriteGroupOfVopHeader(Bitstream * const bs,
+                               const MBParam * pParam,
+                               uint32_t is_closed_gov)
+{
+  int64_t time = (pParam->m_stamp + (pParam->fbase/2)) / pParam->fbase;
+  int hours, minutes, seconds;
+
+  /* compute time_code */
+  seconds = time % 60; time /= 60;
+  minutes = time % 60; time /= 60;
+  hours = time % 24; /* don't overflow */
+      
+  BitstreamPutBits(bs, GRPOFVOP_START_CODE, 32);
+  BitstreamPutBits(bs, hours, 5);
+  BitstreamPutBits(bs, minutes, 6);
+  BitstreamPutBit(bs, 1);
+  BitstreamPutBits(bs, seconds, 6);
+  BitstreamPutBits(bs, is_closed_gov, 1);
+  BitstreamPutBits(bs, 0, 1); /* broken_link */
+}
+
+/*
+ * End of Sequence
+ */
+void
+BitstreamWriteEndOfSequence(Bitstream * const bs)
+{
+    BitstreamPadAlways(bs);
+    BitstreamPutBits(bs, VISOBJSEQ_STOP_CODE, 32);
+}
+
+/*
+ * Video Packet (resync marker)
+ */
+
+void write_video_packet_header(Bitstream * const bs,
+                               const MBParam * pParam,
+                               const FRAMEINFO * const frame,
+                               int mbnum)
+{
+    const int mbnum_bits = log2bin(pParam->mb_width *  pParam->mb_height - 1);
+    uint32_t nbitsresyncmarker;
+
+    if (frame->coding_type == I_VOP)
+      nbitsresyncmarker = NUMBITS_VP_RESYNC_MARKER;  /* 16 zeros followed by a 1. */
+    else if (frame->coding_type == P_VOP)
+      nbitsresyncmarker = NUMBITS_VP_RESYNC_MARKER-1 + frame->fcode;
+    else /* B_VOP */
+      nbitsresyncmarker = MAX(NUMBITS_VP_RESYNC_MARKER, NUMBITS_VP_RESYNC_MARKER-1 + MAX(frame->fcode, frame->bcode));
+
+    BitstreamPadAlways(bs);
+    BitstreamPutBits(bs, RESYNC_MARKER, nbitsresyncmarker);
+    BitstreamPutBits(bs, mbnum, mbnum_bits);
+    BitstreamPutBits(bs, frame->quant, 5);
+    BitstreamPutBit(bs, 0); /* hec */
+}
