@@ -20,7 +20,7 @@
  *  along with this program ; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * $Id$
+ * $Id: estimation_rd_based.c,v 1.13 2005/11/22 10:23:01 suxen_drol Exp $
  *
  ****************************************************************************/
 
@@ -56,13 +56,15 @@ Block_CalcBits(	int16_t * const coeff,
 				uint32_t * cbp,
 				const int block,
 				const uint16_t * scan_table,
-				const uint16_t * mpeg_quant_matrices)
+				const unsigned int lambda,
+				const uint16_t * mpeg_quant_matrices,
+				const unsigned int quant_sq)
 {
 	int sum;
 	int bits;
 	int distortion = 0;
 
-	fdct(data);
+	fdct((short * const)data);
 
 	if (quant_type) sum = quant_h263_inter(coeff, data, quant, mpeg_quant_matrices);
 	else sum = quant_mpeg_inter(coeff, data, quant, mpeg_quant_matrices);
@@ -92,7 +94,7 @@ Block_CalcBits(	int16_t * const coeff,
 	}
 
 
-	return bits + (LAMBDA*distortion)/(quant*quant);
+	return bits + (lambda*distortion)/quant_sq;
 }
 
 static __inline unsigned int
@@ -109,7 +111,9 @@ Block_CalcBitsIntra(MACROBLOCK * pMB,
 					const int quant_type,
 					unsigned int bits[2],
 					unsigned int cbp[2],
-					const uint16_t * mpeg_quant_matrices)
+					unsigned int lambda,
+					const uint16_t * mpeg_quant_matrices,
+					const unsigned int quant_sq)
 {
 	int direction;
 	int16_t *pCurrent;
@@ -117,7 +121,7 @@ Block_CalcBitsIntra(MACROBLOCK * pMB,
 	unsigned int distortion = 0;
 	const uint32_t iDcScaler = get_dc_scaler(quant, block < 4);
 
-	fdct(coeff);
+	fdct((short * const)coeff);
 
 	if (quant_type) {
 		quant_h263_intra(qcoeff, coeff, quant, iDcScaler, mpeg_quant_matrices);
@@ -131,7 +135,7 @@ Block_CalcBitsIntra(MACROBLOCK * pMB,
 				quant, iDcScaler, predictors, 0);
 	
 	direction = pMB->acpred_directions[block];
-	pCurrent = pMB->pred_values[block];
+	pCurrent = (int16_t*)pMB->pred_values[block];
 
 	/* store current coeffs to pred_values[] for future prediction */
 	pCurrent[0] = qcoeff[0] * iDcScaler;
@@ -144,8 +148,8 @@ Block_CalcBitsIntra(MACROBLOCK * pMB,
 	/* dc prediction */
 	qcoeff[0] = qcoeff[0] - predictors[0];
 
-	if (block < 4) bits[1] = bits[0] = dcy_tab[qcoeff[0] + 255].len;
-	else bits[1] = bits[0] = dcc_tab[qcoeff[0] + 255].len;
+	if (block < 4) bits[1] = bits[0] = dcy_tab[qcoeff[0] + 255].len - 3; /* 3 bits added before (4 times) */
+	else bits[1] = bits[0] = dcc_tab[qcoeff[0] + 255].len - 2; /* 2 bits added before (2 times)*/
 
 	/* calc cost before ac prediction */
 	bits[0] += coded = CodeCoeffIntra_CalcBits(qcoeff, scan_tables[0]);
@@ -169,7 +173,7 @@ Block_CalcBitsIntra(MACROBLOCK * pMB,
 
 	distortion = sse8_16bit(coeff, dqcoeff, 8*sizeof(int16_t));
 
-	return (LAMBDA*distortion)/(quant*quant);
+	return (lambda*distortion)/quant_sq;
 }
 
 
@@ -179,7 +183,8 @@ CheckCandidateRD16(const int x, const int y, SearchData * const data, const unsi
 {
 
 	int16_t *in = data->dctSpace, *coeff = data->dctSpace + 64;
-	int32_t rd = 0;
+	/* minimum nuber of bits INTER can take is 1 (mcbpc) + 2 (cby) + 2 (vector) */
+	int32_t rd = BITS_MULT * (1+2+2);
 	VECTOR * current;
 	const uint8_t * ptr;
 	int i, t, xc, yc;
@@ -201,10 +206,12 @@ CheckCandidateRD16(const int x, const int y, SearchData * const data, const unsi
 	for(i = 0; i < 4; i++) {
 		int s = 8*((i&1) + (i>>1)*data->iEdgedWidth);
 		transfer_8to16subro(in, data->Cur + s, ptr + s, data->iEdgedWidth);
-		rd += data->temp[i] = Block_CalcBits(coeff, in, data->dctSpace + 128, data->iQuant, data->quant_type, &cbp, i, data->scan_table, data->mpeg_quant_matrices);
+		rd += data->temp[i] = Block_CalcBits(coeff, in, data->dctSpace + 128, data->iQuant, 
+								data->quant_type, &cbp, i, data->scan_table, data->lambda[i], 
+								data->mpeg_quant_matrices, data->quant_sq);
 	}
 
-	rd += t = BITS_MULT*d_mv_bits(x, y, data->predMV, data->iFcode, data->qpel^data->qpel_precision, 0);
+	rd += t = BITS_MULT * (d_mv_bits(x, y, data->predMV, data->iFcode, data->qpel^data->qpel_precision) - 2);
 
 	if (data->temp[0] + t < data->iMinSAD[1]) {
 		data->iMinSAD[1] = data->temp[0] + t; current[1].x = x; current[1].y = y; data->cbp[1] = (data->cbp[1]&~32) | (cbp&32); }
@@ -215,7 +222,7 @@ CheckCandidateRD16(const int x, const int y, SearchData * const data, const unsi
 	if (data->temp[3] < data->iMinSAD[4]) {
 		data->iMinSAD[4] = data->temp[3]; current[4].x = x; current[4].y = y; data->cbp[1] = (data->cbp[1]&~4) | (cbp&4); }
 
-	rd += BITS_MULT*xvid_cbpy_tab[15-(cbp>>2)].len;
+	rd += BITS_MULT * (xvid_cbpy_tab[15-(cbp>>2)].len - 2);
 
 	if (rd >= data->iMinSAD[0]) return;
 
@@ -226,15 +233,19 @@ CheckCandidateRD16(const int x, const int y, SearchData * const data, const unsi
 	/* chroma U */
 	ptr = interpolate8x8_switch2(data->RefQ, data->RefP[4], 0, 0, xc, yc, data->iEdgedWidth/2, data->rounding);
 	transfer_8to16subro(in, data->CurU, ptr, data->iEdgedWidth/2);
-	rd += Block_CalcBits(coeff, in, data->dctSpace + 128, data->iQuant, data->quant_type, &cbp, 4, data->scan_table, data->mpeg_quant_matrices);
+	rd += Block_CalcBits(coeff, in, data->dctSpace + 128, data->iQuant, data->quant_type,
+							&cbp, 4, data->scan_table, data->lambda[4], 
+							data->mpeg_quant_matrices, data->quant_sq);
 	if (rd >= data->iMinSAD[0]) return;
 
 	/* chroma V */
 	ptr = interpolate8x8_switch2(data->RefQ, data->RefP[5], 0, 0, xc, yc, data->iEdgedWidth/2, data->rounding);
 	transfer_8to16subro(in, data->CurV, ptr, data->iEdgedWidth/2);
-	rd += Block_CalcBits(coeff, in, data->dctSpace + 128, data->iQuant, data->quant_type, &cbp, 5, data->scan_table, data->mpeg_quant_matrices);
+	rd += Block_CalcBits(coeff, in, data->dctSpace + 128, data->iQuant, data->quant_type,
+							&cbp, 5, data->scan_table, data->lambda[5], 
+							data->mpeg_quant_matrices, data->quant_sq);
 
-	rd += BITS_MULT*mcbpc_inter_tab[(MODE_INTER & 7) | ((cbp & 3) << 3)].len;
+	rd += BITS_MULT * (mcbpc_inter_tab[(MODE_INTER & 7) | ((cbp & 3) << 3)].len - 1); /* one was added before */
 
 	if (rd < data->iMinSAD[0]) {
 		data->iMinSAD[0] = rd;
@@ -266,8 +277,12 @@ CheckCandidateRD8(const int x, const int y, SearchData * const data, const unsig
 	}
 
 	transfer_8to16subro(in, data->Cur, ptr, data->iEdgedWidth);
-	rd = Block_CalcBits(coeff, in, data->dctSpace + 128, data->iQuant, data->quant_type, &cbp, 5, data->scan_table, data->mpeg_quant_matrices);
-	rd += BITS_MULT*d_mv_bits(x, y, data->predMV, data->iFcode, data->qpel^data->qpel_precision, 0);
+	rd = Block_CalcBits(coeff, in, data->dctSpace + 128, data->iQuant, data->quant_type, 
+						&cbp, 5, data->scan_table, data->lambda[0], 
+						data->mpeg_quant_matrices, data->quant_sq);
+	/* we took 2 bits into account before */
+	rd += BITS_MULT * (d_mv_bits(x, y, data->predMV, data->iFcode, data->qpel^data->qpel_precision) - 2); 
+
 
 	if (rd < data->iMinSAD[0]) {
 		*data->cbp = cbp;
@@ -298,7 +313,7 @@ findRD_inter(SearchData * const Data,
 		if (MotionFlags & (XVID_ME_HALFPELREFINE16_RD | XVID_ME_EXTSEARCH_RD)) { /* we have to prepare for halfpixel-precision search */
 			for(i = 0; i < 5; i++) bsad[i] = Data->iMinSAD[i];
 			get_range(&Data->min_dx, &Data->max_dx, &Data->min_dy, &Data->max_dy, x, y, 4,
-						pParam->width, pParam->height, Data->iFcode - Data->qpel, 1, Data->rrv);
+						pParam->width, pParam->height, Data->iFcode - Data->qpel, 1);
 			Data->qpel_precision = 0;
 			if (Data->currentQMV->x & 1 || Data->currentQMV->y & 1)
 				CheckCandidateRD16(Data->currentMV[0].x, Data->currentMV[0].y, Data, 255);
@@ -313,7 +328,7 @@ findRD_inter(SearchData * const Data,
 		xvid_me_SquareSearch(Data->currentMV->x, Data->currentMV->y, Data, 255, CheckCandidateRD16);
 
 	if (MotionFlags&XVID_ME_HALFPELREFINE16_RD)
-		xvid_me_SubpelRefine(Data, CheckCandidateRD16);
+		xvid_me_SubpelRefine(Data->currentMV[0], Data, CheckCandidateRD16, 0);
 
 	if (Data->qpel) {
 		if (MotionFlags&(XVID_ME_EXTSEARCH_RD | XVID_ME_HALFPELREFINE16_RD)) { /* there was halfpel-precision search */
@@ -325,10 +340,14 @@ findRD_inter(SearchData * const Data,
 			/* preparing for qpel-precision search */
 			Data->qpel_precision = 1;
 			get_range(&Data->min_dx, &Data->max_dx, &Data->min_dy, &Data->max_dy, x, y, 4,
-					pParam->width, pParam->height, Data->iFcode, 2, 0);
+					pParam->width, pParam->height, Data->iFcode, 2);
+		}
+		if (MotionFlags & XVID_ME_QUARTERPELREFINE16_RD) {
+			if (MotionFlags & XVID_ME_FASTREFINE16)
+				FullRefine_Fast(Data, CheckCandidateRD16, 0);
+			else 
+				xvid_me_SubpelRefine(Data->currentQMV[0], Data, CheckCandidateRD16, 0);
 		}
-		if (MotionFlags&XVID_ME_QUARTERPELREFINE16_RD)
-			xvid_me_SubpelRefine(Data, CheckCandidateRD16);
 	}
 
 	if (MotionFlags&XVID_ME_CHECKPREDICTION_RD) { /* let's check vector equal to prediction */
@@ -347,7 +366,10 @@ findRD_inter4v(SearchData * const Data,
 				const VECTOR * const backup)
 {
 
-	unsigned int cbp = 0, bits = 0, t = 0, i;
+	unsigned int cbp = 0, t = 0, i;
+	
+	/* minimum number of bits INTER4V can take is 2 (cbpy) + 3 (mcbpc) + 4*2 (vectors)*/
+	int bits = (2+3+4*2)*BITS_MULT;
 	SearchData Data2, *Data8 = &Data2;
 	int sumx = 0, sumy = 0;
 	int16_t *in = Data->dctSpace, *coeff = Data->dctSpace + 64;
@@ -366,21 +388,22 @@ findRD_inter4v(SearchData * const Data,
 		Data8->RefP[1] = Data->RefP[1] + 8*((i&1) + (i>>1)*Data->iEdgedWidth);
 		Data8->RefP[3] = Data->RefP[3] + 8*((i&1) + (i>>1)*Data->iEdgedWidth);
 		*Data8->cbp = (Data->cbp[1] & (1<<(5-i))) ? 1:0; /* copy corresponding cbp bit */
+		Data8->lambda[0] = Data->lambda[i];
 
 		if(Data->qpel) {
 			Data8->predMV = get_qpmv2(pMBs, pParam->mb_width, 0, x, y, i);
 			if (i != 0)	t = d_mv_bits(	Data8->currentQMV->x, Data8->currentQMV->y,
-										Data8->predMV, Data8->iFcode, 0, 0);
+										Data8->predMV, Data8->iFcode, 0) - 2;
 		} else {
 			Data8->predMV = get_pmv2(pMBs, pParam->mb_width, 0, x, y, i);
 			if (i != 0)	t = d_mv_bits(	Data8->currentMV->x, Data8->currentMV->y,
-										Data8->predMV, Data8->iFcode, 0, 0);
+										Data8->predMV, Data8->iFcode, 0) - 2;
 		}
 
 		get_range(&Data8->min_dx, &Data8->max_dx, &Data8->min_dy, &Data8->max_dy, 2*x + (i&1), 2*y + (i>>1), 3,
-					pParam->width, pParam->height, Data8->iFcode, Data8->qpel+1, 0);
+					pParam->width, pParam->height, Data8->iFcode, Data8->qpel+1);
 
-		*Data8->iMinSAD += BITS_MULT*t;
+		*Data8->iMinSAD += BITS_MULT * t;
 
 		Data8->qpel_precision = Data8->qpel;
 		/* checking the vector which has been found by SAD-based 8x8 search (if it's different than the one found so far) */
@@ -391,13 +414,17 @@ findRD_inter4v(SearchData * const Data,
 		}
 
 		if (Data8->qpel) {
+			int bsad = Data8->iMinSAD[0];
+			int bx = Data8->currentQMV->x;
+			int by = Data8->currentQMV->y;
+
+			Data8->currentMV->x = Data8->currentQMV->x/2;
+			Data8->currentMV->y = Data8->currentQMV->y/2;
+
 			if (MotionFlags&XVID_ME_HALFPELREFINE8_RD || (MotionFlags&XVID_ME_EXTSEARCH8 && MotionFlags&XVID_ME_EXTSEARCH_RD)) { /* halfpixel motion search follows */
-				int32_t s = *Data8->iMinSAD;
-				Data8->currentMV->x = Data8->currentQMV->x/2;
-				Data8->currentMV->y = Data8->currentQMV->y/2;
 				Data8->qpel_precision = 0;
 				get_range(&Data8->min_dx, &Data8->max_dx, &Data8->min_dy, &Data8->max_dy, 2*x + (i&1), 2*y + (i>>1), 3,
-							pParam->width, pParam->height, Data8->iFcode - 1, 1, 0);
+							pParam->width, pParam->height, Data8->iFcode - 1, 1);
 
 				if (Data8->currentQMV->x & 1 || Data8->currentQMV->y & 1)
 					CheckCandidateRD8(Data8->currentMV->x, Data8->currentMV->y, Data8, 255);
@@ -406,20 +433,32 @@ findRD_inter4v(SearchData * const Data,
 					xvid_me_SquareSearch(Data8->currentMV->x, Data8->currentMV->x, Data8, 255, CheckCandidateRD8);
 
 				if (MotionFlags & XVID_ME_HALFPELREFINE8_RD)
-					xvid_me_SubpelRefine(Data8, CheckCandidateRD8);
+					xvid_me_SubpelRefine(Data->currentMV[0], Data8, CheckCandidateRD8, 0);
 
-				if(s > *Data8->iMinSAD) { /* we have found a better match */
-					Data8->currentQMV->x = 2*Data8->currentMV->x;
-					Data8->currentQMV->y = 2*Data8->currentMV->y;
+				if(bsad > *Data8->iMinSAD) { /* we have found a better match */
+					bx = Data8->currentQMV->x = 2*Data8->currentMV->x;
+					by = Data8->currentQMV->y = 2*Data8->currentMV->y;
+					bsad = Data8->iMinSAD[0];
 				}
 
 				Data8->qpel_precision = 1;
 				get_range(&Data8->min_dx, &Data8->max_dx, &Data8->min_dy, &Data8->max_dy, 2*x + (i&1), 2*y + (i>>1), 3,
-							pParam->width, pParam->height, Data8->iFcode, 2, 0);
+							pParam->width, pParam->height, Data8->iFcode, 2);
 
 			}
-			if (MotionFlags & XVID_ME_QUARTERPELREFINE8_RD)
-				xvid_me_SubpelRefine(Data8, CheckCandidateRD8);
+
+			if (MotionFlags & XVID_ME_QUARTERPELREFINE8_RD) {
+				if (MotionFlags & XVID_ME_FASTREFINE8)
+					FullRefine_Fast(Data8, CheckCandidateRD8, 0);
+				else xvid_me_SubpelRefine(Data->currentQMV[0], Data8, CheckCandidateRD8, 0);
+			}
+
+			if (bsad <= Data->iMinSAD[0]) { 
+				/* we have not found a better match */
+				Data8->iMinSAD[0] = bsad;
+				Data8->currentQMV->x = bx;
+				Data8->currentQMV->y = by;
+			}
 
 		} else { /* not qpel */
 
@@ -427,7 +466,7 @@ findRD_inter4v(SearchData * const Data,
 				xvid_me_SquareSearch(Data8->currentMV->x, Data8->currentMV->x, Data8, 255, CheckCandidateRD8);
 
 			if (MotionFlags & XVID_ME_HALFPELREFINE8_RD)
-				xvid_me_SubpelRefine(Data8, CheckCandidateRD8); /* halfpel refinement */
+				xvid_me_SubpelRefine(Data->currentMV[0], Data8, CheckCandidateRD8, 0); /* halfpel refinement */
 		}
 
 		/* checking vector equal to predicion */
@@ -459,7 +498,7 @@ findRD_inter4v(SearchData * const Data,
 
 	} /* end - for all luma blocks */
 
-	bits += BITS_MULT*xvid_cbpy_tab[15-(cbp>>2)].len;
+	bits += BITS_MULT * (xvid_cbpy_tab[15-(cbp>>2)].len - 2); /* 2 were added before */
 
 	/* let's check chroma */
 	sumx = (sumx >> 3) + roundtab_76[sumx & 0xf];
@@ -468,16 +507,18 @@ findRD_inter4v(SearchData * const Data,
 	/* chroma U */
 	ptr = interpolate8x8_switch2(Data->RefQ + 64, Data->RefP[4], 0, 0, sumx, sumy, Data->iEdgedWidth/2, Data->rounding);
 	transfer_8to16subro(in, Data->CurU, ptr, Data->iEdgedWidth/2);
-	bits += Block_CalcBits(coeff, in, Data->dctSpace + 128, Data->iQuant, Data->quant_type, &cbp, 4, Data->scan_table, Data->mpeg_quant_matrices);
+	bits += Block_CalcBits(coeff, in, Data->dctSpace + 128, Data->iQuant, Data->quant_type, &cbp, 4,
+							Data->scan_table, Data->lambda[4], Data->mpeg_quant_matrices, Data->quant_sq);
 
 	if (bits >= *Data->iMinSAD) return bits;
 
 	/* chroma V */
 	ptr = interpolate8x8_switch2(Data->RefQ + 64, Data->RefP[5], 0, 0, sumx, sumy, Data->iEdgedWidth/2, Data->rounding);
 	transfer_8to16subro(in, Data->CurV, ptr, Data->iEdgedWidth/2);
-	bits += Block_CalcBits(coeff, in, Data->dctSpace + 128, Data->iQuant, Data->quant_type, &cbp, 5, Data->scan_table, Data->mpeg_quant_matrices);
+	bits += Block_CalcBits(coeff, in, Data->dctSpace + 128, Data->iQuant, Data->quant_type, &cbp, 5,
+							Data->scan_table, Data->lambda[5], Data->mpeg_quant_matrices, Data->quant_sq);
 
-	bits += BITS_MULT*mcbpc_inter_tab[(MODE_INTER4V & 7) | ((cbp & 3) << 3)].len;
+	bits += BITS_MULT*(mcbpc_inter_tab[(MODE_INTER4V & 7) | ((cbp & 3) << 3)].len - 3); /* 3 were added before */
 
 	*Data->cbp = cbp;
 	return bits;
@@ -488,7 +529,8 @@ findRD_intra(SearchData * const Data, MACROBLOCK * pMB,
 			 const int x, const int y, const int mb_width)
 {
 	unsigned int cbp[2] = {0, 0}, bits[2], i;
-	unsigned int bits1 = BITS_MULT*1, bits2 = BITS_MULT*1; /* this one is ac/dc prediction flag bit */
+	/* minimum number of bits that WILL be coded in intra - mcbpc 5, cby 2 acdc flag - 1 and DC coeffs - 4*3+2*2 */
+	int bits1 = BITS_MULT*(5+2+1+4*3+2*2), bits2 = BITS_MULT*(5+2+1+4*3+2*2);
 	unsigned int distortion = 0;
 
 	int16_t *in = Data->dctSpace, * coeff = Data->dctSpace + 64, * dqcoeff = Data->dctSpace + 128;
@@ -501,7 +543,8 @@ findRD_intra(SearchData * const Data, MACROBLOCK * pMB,
 		
 
 		distortion = Block_CalcBitsIntra(pMB, x, y, mb_width, i, in, coeff, dqcoeff,
-								predictors[i], iQuant, Data->quant_type, bits, cbp, Data->mpeg_quant_matrices);
+								predictors[i], iQuant, Data->quant_type, bits, cbp, 
+								Data->lambda[i], Data->mpeg_quant_matrices, Data->quant_sq);
 		bits1 += distortion + BITS_MULT * bits[0];
 		bits2 += distortion + BITS_MULT * bits[1];
 
@@ -509,13 +552,14 @@ findRD_intra(SearchData * const Data, MACROBLOCK * pMB,
 			return bits1;
 	}
 
-	bits1 += BITS_MULT*xvid_cbpy_tab[cbp[0]>>2].len;
-	bits2 += BITS_MULT*xvid_cbpy_tab[cbp[1]>>2].len;
+	bits1 += BITS_MULT * (xvid_cbpy_tab[cbp[0]>>2].len - 2); /* two bits were added before */
+	bits2 += BITS_MULT * (xvid_cbpy_tab[cbp[1]>>2].len - 2);
 
 	/*chroma U */
 	transfer_8to16copy(in, Data->CurU, Data->iEdgedWidth/2);
 	distortion = Block_CalcBitsIntra(pMB, x, y, mb_width, 4, in, coeff, dqcoeff,
-									predictors[4], iQuant, Data->quant_type, bits, cbp, Data->mpeg_quant_matrices);
+									predictors[4], iQuant, Data->quant_type, bits, cbp, 
+									Data->lambda[4], Data->mpeg_quant_matrices, Data->quant_sq);
 	bits1 += distortion + BITS_MULT * bits[0];
 	bits2 += distortion + BITS_MULT * bits[1];
 
@@ -525,13 +569,14 @@ findRD_intra(SearchData * const Data, MACROBLOCK * pMB,
 	/* chroma V */
 	transfer_8to16copy(in, Data->CurV, Data->iEdgedWidth/2);
 	distortion = Block_CalcBitsIntra(pMB, x, y, mb_width, 5, in, coeff, dqcoeff,
-									predictors[5], iQuant, Data->quant_type, bits, cbp, Data->mpeg_quant_matrices);
+									predictors[5], iQuant, Data->quant_type, bits, cbp, 
+									Data->lambda[5], Data->mpeg_quant_matrices, Data->quant_sq);
 
 	bits1 += distortion + BITS_MULT * bits[0];
 	bits2 += distortion + BITS_MULT * bits[1];
 
-	bits1 += BITS_MULT*mcbpc_inter_tab[(MODE_INTRA & 7) | ((cbp[0] & 3) << 3)].len;
-	bits2 += BITS_MULT*mcbpc_inter_tab[(MODE_INTRA & 7) | ((cbp[1] & 3) << 3)].len;
+	bits1 += BITS_MULT * (mcbpc_inter_tab[(MODE_INTRA & 7) | ((cbp[0] & 3) << 3)].len - 5); /* 5 bits were added before */
+	bits2 += BITS_MULT * (mcbpc_inter_tab[(MODE_INTRA & 7) | ((cbp[1] & 3) << 3)].len - 5);
 
 	*Data->cbp = bits1 <= bits2 ? cbp[0] : cbp[1];
 
@@ -542,30 +587,34 @@ findRD_intra(SearchData * const Data, MACROBLOCK * pMB,
 static int
 findRD_gmc(SearchData * const Data, const IMAGE * const vGMC, const int x, const int y)
 {
-	int bits = BITS_MULT*1; /* this one is mcsel */
+	/* minimum nubler of bits - 1 (mcbpc) + 2 (cby) + 1 (mcsel) */
+	int bits = BITS_MULT * (1+2+1);
 	unsigned int cbp = 0, i;
 	int16_t *in = Data->dctSpace, * coeff = Data->dctSpace + 64;
 
 	for(i = 0; i < 4; i++) {
 		int s = 8*((i&1) + (i>>1)*Data->iEdgedWidth);
 		transfer_8to16subro(in, Data->Cur + s, vGMC->y + s + 16*(x+y*Data->iEdgedWidth), Data->iEdgedWidth);
-		bits += Block_CalcBits(coeff, in, Data->dctSpace + 128, Data->iQuant, Data->quant_type, &cbp, i, Data->scan_table, Data->mpeg_quant_matrices);
+		bits += Block_CalcBits(coeff, in, Data->dctSpace + 128, Data->iQuant, Data->quant_type, &cbp, i,
+								Data->scan_table, Data->lambda[i], Data->mpeg_quant_matrices, Data->quant_sq);
 		if (bits >= Data->iMinSAD[0]) return bits;
 	}
 
-	bits += BITS_MULT*xvid_cbpy_tab[15-(cbp>>2)].len;
+	bits += BITS_MULT * (xvid_cbpy_tab[15-(cbp>>2)].len - 2);
 
 	/*chroma U */
 	transfer_8to16subro(in, Data->CurU, vGMC->u + 8*(x+y*(Data->iEdgedWidth/2)), Data->iEdgedWidth/2);
-	bits += Block_CalcBits(coeff, in, Data->dctSpace + 128, Data->iQuant, Data->quant_type, &cbp, 4, Data->scan_table, Data->mpeg_quant_matrices);
+	bits += Block_CalcBits(coeff, in, Data->dctSpace + 128, Data->iQuant, Data->quant_type, &cbp, 4,
+							Data->scan_table, Data->lambda[4], Data->mpeg_quant_matrices, Data->quant_sq);
 
 	if (bits >= Data->iMinSAD[0]) return bits;
 
 	/* chroma V */
 	transfer_8to16subro(in, Data->CurV , vGMC->v + 8*(x+y*(Data->iEdgedWidth/2)), Data->iEdgedWidth/2);
-	bits += Block_CalcBits(coeff, in, Data->dctSpace + 128, Data->iQuant, Data->quant_type, &cbp, 5, Data->scan_table, Data->mpeg_quant_matrices);
+	bits += Block_CalcBits(coeff, in, Data->dctSpace + 128, Data->iQuant, Data->quant_type, &cbp, 5,
+							Data->scan_table, Data->lambda[5], Data->mpeg_quant_matrices, Data->quant_sq);
 
-	bits += BITS_MULT*mcbpc_inter_tab[(MODE_INTER & 7) | ((cbp & 3) << 3)].len;
+	bits += BITS_MULT * (mcbpc_inter_tab[(MODE_INTER & 7) | ((cbp & 3) << 3)].len - 1);
 
 	*Data->cbp = cbp;
 
@@ -590,10 +639,10 @@ xvid_me_ModeDecision_RD(SearchData * const Data,
 	int mcsel = 0;
 	int inter4v = (VopFlags & XVID_VOP_INTER4V) && (pMB->dquant == 0);
 	const uint32_t iQuant = pMB->quant;
-
 	int min_rd, intra_rd, i, cbp;
 	VECTOR backup[5], *v;
 	Data->iQuant = iQuant;
+	Data->quant_sq = iQuant*iQuant;
 	Data->scan_table = VopFlags & XVID_VOP_ALTERNATESCAN ?
 						scan_tables[2] : scan_tables[0];
 
@@ -605,6 +654,11 @@ xvid_me_ModeDecision_RD(SearchData * const Data,
 		backup[i] = v[i];
 	}
 
+	for (i = 0; i < 6; i++) {
+		/* HVS models, anyone ? */
+		Data->lambda[i] = LAMBDA;
+	}
+
 	min_rd = findRD_inter(Data, x, y, pParam, MotionFlags);
 	cbp = *Data->cbp;
 
@@ -629,23 +683,20 @@ xvid_me_ModeDecision_RD(SearchData * const Data,
 			cbp = *Data->cbp;
 		}
 	}
-
-	intra_rd = findRD_intra(Data, pMB, x, y, pParam->mb_width);
-	if (intra_rd < min_rd) {
-		*Data->iMinSAD = min_rd = intra_rd;
-		mode = MODE_INTRA;
-		cbp = *Data->cbp;
+	
+	/* there is no way for INTRA to take less than 24 bits - go to findRD_intra() for calculations */
+	if (min_rd > 24*BITS_MULT) { 
+		intra_rd = findRD_intra(Data, pMB, x, y, pParam->mb_width);
+		if (intra_rd < min_rd) {
+			*Data->iMinSAD = min_rd = intra_rd;
+			mode = MODE_INTRA;
+			cbp = *Data->cbp;
+		}
 	}
 
 	pMB->sad16 = pMB->sad8[0] = pMB->sad8[1] = pMB->sad8[2] = pMB->sad8[3] = 0;
 	pMB->cbp = cbp;
 
-
-	if (Data->rrv) {
-			Data->currentMV[0].x = RRV_MV_SCALEDOWN(Data->currentMV[0].x);
-			Data->currentMV[0].y = RRV_MV_SCALEDOWN(Data->currentMV[0].y);
-	}
-
 	if (mode == MODE_INTER && mcsel == 0) {
 		pMB->mvs[0] = pMB->mvs[1] = pMB->mvs[2] = pMB->mvs[3] = Data->currentMV[0];
 
@@ -707,6 +758,13 @@ xvid_me_ModeDecision_Fast(SearchData * const Data,
 						scan_tables[2] : scan_tables[0];
 
 	pMB->mcsel = 0;
+	Data->iQuant = iQuant;
+	Data->quant_sq = iQuant*iQuant;
+
+	for (i = 0; i < 6; i++) {
+		/* HVS models, anyone ? */
+		Data->lambda[i] = LAMBDA;
+	}
 
 	/* INTER <-> INTER4V decision */
 	if ((Data->iMinSAD[0] + 75 < Data->iMinSAD[1] +
@@ -725,7 +783,7 @@ xvid_me_ModeDecision_Fast(SearchData * const Data,
 		/* final skip decision, a.k.a. "the vector you found, really that good?" */
 		if (skip_possible && (pMB->sad16 < (int)iQuant * MAX_SAD00_FOR_SKIP))
 			if ( (100*sad)/(pMB->sad16+1) > FINAL_SKIP_THRESH)
-				if (Data->chroma || xvid_me_SkipDecisionP(pCurrent, pRef, x, y, Data->iEdgedWidth/2, iQuant, Data->rrv)) {
+				if (Data->chroma || xvid_me_SkipDecisionP(pCurrent, pRef, x, y, Data->iEdgedWidth/2, iQuant)) {
 					mode = MODE_NOT_CODED;
 					sad = 0;  /* Compiler warning */
 					goto early_out;
@@ -756,7 +814,7 @@ xvid_me_ModeDecision_Fast(SearchData * const Data,
 		/* final skip decision, a.k.a. "the vector you found, really that good?" */
 		if (skip_possible && (pMB->sad16 < (int)iQuant * MAX_SAD00_FOR_SKIP))
 			if ( (100*Data->iMinSAD[0])/(pMB->sad16+1) > FINAL_SKIP_THRESH)
-				if (Data->chroma || xvid_me_SkipDecisionP(pCurrent, pRef, x, y, Data->iEdgedWidth/2, iQuant, Data->rrv)) {
+				if (Data->chroma || xvid_me_SkipDecisionP(pCurrent, pRef, x, y, Data->iEdgedWidth/2, iQuant)) {
 					mode = MODE_NOT_CODED;
 					sad = 0; /* Compiler warning */
 					goto early_out;
@@ -832,18 +890,9 @@ xvid_me_ModeDecision_Fast(SearchData * const Data,
 			if ((pMB - 1)->mode == MODE_INTRA ) InterBias -= 80;
 
 		if (Data->chroma) InterBias += 50; /* dev8(chroma) ??? <-- yes, we need dev8 (no big difference though) */
-		if (Data->rrv) InterBias *= 4;
 
 		if (InterBias < sad) {
-			int32_t deviation;
-			if (!Data->rrv)
-				deviation = dev16(Data->Cur, Data->iEdgedWidth);
-			else
-				deviation = dev16(Data->Cur, Data->iEdgedWidth) + /* dev32() */
-							dev16(Data->Cur+16, Data->iEdgedWidth) +
-							dev16(Data->Cur + 16*Data->iEdgedWidth, Data->iEdgedWidth) +
-							dev16(Data->Cur+16+16*Data->iEdgedWidth, Data->iEdgedWidth);
-
+			int32_t deviation = dev16(Data->Cur, Data->iEdgedWidth);
 			if (deviation < (sad - InterBias)) mode = MODE_INTRA;
 		}
 
@@ -899,11 +948,6 @@ xvid_me_ModeDecision_Fast(SearchData * const Data,
 early_out:
 	pMB->sad16 = pMB->sad8[0] = pMB->sad8[1] = pMB->sad8[2] = pMB->sad8[3] = sad;
 
-	if (Data->rrv) {
-			Data->currentMV[0].x = RRV_MV_SCALEDOWN(Data->currentMV[0].x);
-			Data->currentMV[0].y = RRV_MV_SCALEDOWN(Data->currentMV[0].y);
-	}
-
 	if (mode == MODE_INTER && mcsel == 0) {
 		pMB->mvs[0] = pMB->mvs[1] = pMB->mvs[2] = pMB->mvs[3] = Data->currentMV[0];
 
