@@ -3,7 +3,7 @@
  *  XVID MPEG-4 VIDEO CODEC
  *  - Native API implementation  -
  *
- *  Copyright(C) 2001-2003 Peter Ross <pross@xvid.org>
+ *  Copyright(C) 2001-2004 Peter Ross <pross@xvid.org>
  *
  *  This program is free software ; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *  along with this program ; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * $Id$
+ * $Id: xvid.c,v 1.65.2.4 2006/11/01 10:17:27 Isibaar Exp $
  *
  ****************************************************************************/
 
@@ -36,7 +36,6 @@
 #include "dct/fdct.h"
 #include "image/colorspace.h"
 #include "image/interpolate8x8.h"
-#include "image/reduced.h"
 #include "utils/mem_transfer.h"
 #include "utils/mbfunctions.h"
 #include "quant/quant.h"
@@ -52,10 +51,9 @@
 unsigned int xvid_debug = 0; /* xvid debug mask */
 #endif
 
-#if defined(ARCH_IS_IA32)
-#if defined(_MSC_VER)
+#if (defined(ARCH_IS_IA32) || defined(ARCH_IS_X86_64)) && defined(_MSC_VER)
 #	include <windows.h>
-#else
+#elif defined(ARCH_IS_IA32) || defined(ARCH_IS_X86_64) || defined(ARCH_IS_PPC)
 #	include <signal.h>
 #	include <setjmp.h>
 
@@ -78,53 +76,62 @@ unsigned int xvid_debug = 0; /* xvid debug mask */
  *   0 : SIGILL was *not* signalled
  *   1 : SIGILL was signalled
  */
-
-int
+#if (defined(ARCH_IS_IA32) || defined(ARCH_IS_X86_64)) && defined(_MSC_VER)
+static int
 sigill_check(void (*func)())
 {
-#if defined(_MSC_VER)
 	_try {
 		func();
-	}
-	_except(EXCEPTION_EXECUTE_HANDLER) {
+	} _except(EXCEPTION_EXECUTE_HANDLER) {
 
 		if (_exception_code() == STATUS_ILLEGAL_INSTRUCTION)
-			return 1;
+			return(1);
 	}
-	return 0;
-#else
-    void * old_handler;
+	return(0);
+}
+#elif defined(ARCH_IS_IA32) || defined(ARCH_IS_X86_64) || defined(ARCH_IS_PPC)
+static int
+sigill_check(void (*func)())
+{
+    void *old_handler;
     int jmpret;
 
-
+    /* Set our SIGILL handler */
     old_handler = signal(SIGILL, sigill_handler);
-    if (old_handler == SIG_ERR)
-    {
-        return -1;
+
+    /* Check for error */
+    if (old_handler == SIG_ERR) {
+        return(-1);
     }
 
-    jmpret = setjmp(mark);
-    if (jmpret == 0)
-    {
+    /* Save stack context, so if func triggers a SIGILL, we can still roll
+	 * back to a valid CPU state */
+	jmpret = setjmp(mark);
+
+	/* If setjmp returned directly, then its returned value is 0, and we still
+	 * have to test the passed func. Otherwise it means the stack context has
+	 * been restored by a longjmp() call, which in our case happens only in the
+	 * signal handler */
+    if (jmpret == 0) {
         func();
     }
 
+    /* Restore old signal handler */
     signal(SIGILL, old_handler);
 
-    return jmpret;
-#endif
+    return(jmpret);
 }
 #endif
 
 
 /* detect cpu flags  */
 static unsigned int
-detect_cpu_flags()
+detect_cpu_flags(void)
 {
 	/* enable native assembly optimizations by default */
 	unsigned int cpu_flags = XVID_CPU_ASM;
 
-#if defined(ARCH_IS_IA32)
+#if defined(ARCH_IS_IA32) || defined(ARCH_IS_X86_64)
 	cpu_flags |= check_cpu_features();
 	if ((cpu_flags & XVID_CPU_SSE) && sigill_check(sse_os_trigger))
 		cpu_flags &= ~XVID_CPU_SSE;
@@ -134,9 +141,8 @@ detect_cpu_flags()
 #endif
 
 #if defined(ARCH_IS_PPC)
-#if defined(ARCH_IS_PPC_ALTIVEC)
-	cpu_flags |= XVID_CPU_ALTIVEC;
-#endif
+	if (!sigill_check(altivec_trigger))
+		cpu_flags |= XVID_CPU_ALTIVEC;
 #endif
 
 	return cpu_flags;
@@ -177,7 +183,7 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
 	idct = idct_int32;
 
 	/* Only needed on PPC Altivec archs */
-	sadInit = 0;
+	sadInit = NULL;
 
 	/* Restore FPU context : emms_c is a nop functions */
 	emms = emms_c;
@@ -204,8 +210,10 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
 	transfer_8to16sub  = transfer_8to16sub_c;
 	transfer_8to16subro  = transfer_8to16subro_c;
 	transfer_8to16sub2 = transfer_8to16sub2_c;
+	transfer_8to16sub2ro = transfer_8to16sub2ro_c;
 	transfer_16to8add  = transfer_16to8add_c;
 	transfer8x8_copy   = transfer8x8_copy_c;
+	transfer8x4_copy   = transfer8x4_copy_c;
 
 	/* Interlacing functions */
 	MBFieldTest = MBFieldTest_c;
@@ -215,6 +223,15 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
 	interpolate8x8_halfpel_v  = interpolate8x8_halfpel_v_c;
 	interpolate8x8_halfpel_hv = interpolate8x8_halfpel_hv_c;
 
+	interpolate8x4_halfpel_h  = interpolate8x4_halfpel_h_c;
+	interpolate8x4_halfpel_v  = interpolate8x4_halfpel_v_c;
+	interpolate8x4_halfpel_hv = interpolate8x4_halfpel_hv_c;
+
+	interpolate8x8_halfpel_add = interpolate8x8_halfpel_add_c;
+	interpolate8x8_halfpel_h_add = interpolate8x8_halfpel_h_add_c;
+	interpolate8x8_halfpel_v_add = interpolate8x8_halfpel_v_add_c;
+	interpolate8x8_halfpel_hv_add = interpolate8x8_halfpel_hv_add_c;
+
 	interpolate16x16_lowpass_h = interpolate16x16_lowpass_h_c;
 	interpolate16x16_lowpass_v = interpolate16x16_lowpass_v_c;
 	interpolate16x16_lowpass_hv = interpolate16x16_lowpass_hv_c;
@@ -229,13 +246,8 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
 	interpolate8x8_avg2 = interpolate8x8_avg2_c;
 	interpolate8x8_avg4 = interpolate8x8_avg4_c;
 
-	/* reduced resolution */
-	copy_upsampled_8x8_16to8 = xvid_Copy_Upsampled_8x8_16To8_C;
-	add_upsampled_8x8_16to8 = xvid_Add_Upsampled_8x8_16To8_C;
-	vfilter_31 = xvid_VFilter_31_C;
-	hfilter_31 = xvid_HFilter_31_C;
-	filter_18x18_to_8x8 = xvid_Filter_18x18_To_8x8_C;
-	filter_diff_18x18_to_8x8 = xvid_Filter_Diff_18x18_To_8x8_C;
+	/* postprocessing */
+	image_brightness = image_brightness_c;
 
 	/* Initialize internal colorspace transformation tables */
 	colorspace_init();
@@ -292,14 +304,10 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
 	dev16      = dev16_c;
 	sad16v	   = sad16v_c;
 	sse8_16bit = sse8_16bit_c;
+	sse8_8bit  = sse8_8bit_c;
 
 #if defined(ARCH_IS_IA32)
 
-	if ((cpu_flags & XVID_CPU_ASM))	{
-		vfilter_31 = xvid_VFilter_31_x86;
-		hfilter_31 = xvid_HFilter_31_x86;
-	}
-
 	if ((cpu_flags & XVID_CPU_MMX) || (cpu_flags & XVID_CPU_MMXEXT) ||
 		(cpu_flags & XVID_CPU_3DNOW) || (cpu_flags & XVID_CPU_3DNOWEXT) ||
 		(cpu_flags & XVID_CPU_SSE) || (cpu_flags & XVID_CPU_SSE2))
@@ -337,6 +345,7 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
 		transfer_8to16sub2 = transfer_8to16sub2_mmx;
 		transfer_16to8add  = transfer_16to8add_mmx;
 		transfer8x8_copy   = transfer8x8_copy_mmx;
+		transfer8x4_copy   = transfer8x4_copy_mmx;
 
 		/* Interlacing Functions */
 		MBFieldTest = MBFieldTest_mmx;
@@ -346,18 +355,23 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
 		interpolate8x8_halfpel_v  = interpolate8x8_halfpel_v_mmx;
 		interpolate8x8_halfpel_hv = interpolate8x8_halfpel_hv_mmx;
 
+		interpolate8x4_halfpel_h  = interpolate8x4_halfpel_h_mmx;
+		interpolate8x4_halfpel_v  = interpolate8x4_halfpel_v_mmx;
+		interpolate8x4_halfpel_hv = interpolate8x4_halfpel_hv_mmx;
+
+		interpolate8x8_halfpel_add = interpolate8x8_halfpel_add_mmx;
+		interpolate8x8_halfpel_h_add = interpolate8x8_halfpel_h_add_mmx;
+		interpolate8x8_halfpel_v_add = interpolate8x8_halfpel_v_add_mmx;
+		interpolate8x8_halfpel_hv_add = interpolate8x8_halfpel_hv_add_mmx;
+
 		interpolate8x8_6tap_lowpass_h = interpolate8x8_6tap_lowpass_h_mmx;
 		interpolate8x8_6tap_lowpass_v = interpolate8x8_6tap_lowpass_v_mmx;
 
 		interpolate8x8_avg2 = interpolate8x8_avg2_mmx;
 		interpolate8x8_avg4 = interpolate8x8_avg4_mmx;
 
-		/* reduced resolution */
-		copy_upsampled_8x8_16to8 = xvid_Copy_Upsampled_8x8_16To8_mmx;
-		add_upsampled_8x8_16to8 = xvid_Add_Upsampled_8x8_16To8_mmx;
-		hfilter_31 = xvid_HFilter_31_mmx;
-		filter_18x18_to_8x8 = xvid_Filter_18x18_To_8x8_mmx;
-		filter_diff_18x18_to_8x8 = xvid_Filter_Diff_18x18_To_8x8_mmx;
+		/* postprocessing */
+		image_brightness = image_brightness_mmx;
 
 		/* image input xxx_to_yv12 related functions */
 		yv12_to_yv12  = yv12_to_yv12_mmx;
@@ -376,14 +390,15 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
 		yv12_to_uyvyi = yv12_to_uyvyi_mmx;
 
 		/* Motion estimation related functions */
-		calc_cbp = calc_cbp_mmx;
-		sad16    = sad16_mmx;
-		sad8     = sad8_mmx;
-		sad16bi = sad16bi_mmx;
-		sad8bi  = sad8bi_mmx;
-		dev16    = dev16_mmx;
-		sad16v	 = sad16v_mmx;
+		calc_cbp   = calc_cbp_mmx;
+		sad16      = sad16_mmx;
+		sad8       = sad8_mmx;
+		sad16bi    = sad16bi_mmx;
+		sad8bi     = sad8bi_mmx;
+		dev16      = dev16_mmx;
+		sad16v	   = sad16v_mmx;
 		sse8_16bit = sse8_16bit_mmx;
+		sse8_8bit  = sse8_8bit_mmx;
 	}
 
 	/* these 3dnow functions are faster than mmx, but slower than xmm. */
@@ -410,10 +425,15 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
 		interpolate8x8_halfpel_h  = interpolate8x8_halfpel_h_xmm;
 		interpolate8x8_halfpel_v  = interpolate8x8_halfpel_v_xmm;
 		interpolate8x8_halfpel_hv = interpolate8x8_halfpel_hv_xmm;
-
-		/* reduced resolution */
-		copy_upsampled_8x8_16to8 = xvid_Copy_Upsampled_8x8_16To8_xmm;
-		add_upsampled_8x8_16to8 = xvid_Add_Upsampled_8x8_16To8_xmm;
+		
+		interpolate8x4_halfpel_h  = interpolate8x4_halfpel_h_xmm;
+		interpolate8x4_halfpel_v  = interpolate8x4_halfpel_v_xmm;
+		interpolate8x4_halfpel_hv = interpolate8x4_halfpel_hv_xmm;
+		
+		interpolate8x8_halfpel_add = interpolate8x8_halfpel_add_xmm;
+		interpolate8x8_halfpel_h_add = interpolate8x8_halfpel_h_add_xmm;
+		interpolate8x8_halfpel_v_add = interpolate8x8_halfpel_v_add_xmm;
+		interpolate8x8_halfpel_hv_add = interpolate8x8_halfpel_hv_add_xmm;
 
 		/* Quantization */
 		quant_mpeg_intra = quant_mpeg_intra_xmm;
@@ -424,6 +444,7 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
 
 		/* Buffer transfer */
 		transfer_8to16sub2 = transfer_8to16sub2_xmm;
+		transfer_8to16sub2ro = transfer_8to16sub2ro_xmm;
 
 		/* Colorspace transformation */
 		yv12_to_yv12  = yv12_to_yv12_xmm;
@@ -445,6 +466,10 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
 		interpolate8x8_halfpel_h = interpolate8x8_halfpel_h_3dn;
 		interpolate8x8_halfpel_v = interpolate8x8_halfpel_v_3dn;
 		interpolate8x8_halfpel_hv = interpolate8x8_halfpel_hv_3dn;
+
+		interpolate8x4_halfpel_h = interpolate8x4_halfpel_h_3dn;
+		interpolate8x4_halfpel_v = interpolate8x4_halfpel_v_3dn;
+		interpolate8x4_halfpel_hv = interpolate8x4_halfpel_hv_3dn;
 	}
 
 	if ((cpu_flags & XVID_CPU_3DNOWEXT)) {
@@ -456,6 +481,7 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
 		transfer_8to16subro =  transfer_8to16subro_3dne;
 		transfer_16to8add = transfer_16to8add_3dne;
 		transfer8x8_copy = transfer8x8_copy_3dne;
+		transfer8x4_copy = transfer8x4_copy_3dne;
 
 		if ((cpu_flags & XVID_CPU_MMXEXT)) {
 			/* Inverse DCT */
@@ -469,6 +495,10 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
 			interpolate8x8_halfpel_v = interpolate8x8_halfpel_v_3dne;
 			interpolate8x8_halfpel_hv = interpolate8x8_halfpel_hv_3dne;
 
+			interpolate8x4_halfpel_h = interpolate8x4_halfpel_h_3dne;
+			interpolate8x4_halfpel_v = interpolate8x4_halfpel_v_3dne;
+			interpolate8x4_halfpel_hv = interpolate8x4_halfpel_hv_3dne;
+
 			/* Quantization */
 			quant_h263_intra = quant_h263_intra_3dne;		/* cmov only */
 			quant_h263_inter = quant_h263_inter_3dne;
@@ -502,9 +532,12 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
 		sad16    = sad16_sse2;
 		dev16    = dev16_sse2;
 
-		/* DCT operators
-		 * no iDCT because it's not "Walken matching" */
+		/* DCT operators */
 		fdct = fdct_sse2_skal;
+    /* idct = idct_sse2_skal; */   /* Is now IEEE1180 and Walken compliant. Disabled until fully tested. */
+
+		/* postprocessing */
+		image_brightness = image_brightness_sse2;
 	}
 #endif /* ARCH_IS_IA32 */
 
@@ -535,20 +568,132 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
 #endif
 
 #if defined(ARCH_IS_PPC)
-	if ((cpu_flags & XVID_CPU_ASM))
-	{
-		calc_cbp = calc_cbp_ppc;
-	}
+	if ((cpu_flags & XVID_CPU_ALTIVEC)) {
+          /* sad operators */
+		  sad16 = sad16_altivec_c;
+		  sad16bi = sad16bi_altivec_c;
+		  sad8 = sad8_altivec_c;
+		  dev16 = dev16_altivec_c;
+          
+          sse8_16bit = sse8_16bit_altivec_c;
+          
+          /* mem transfer */
+          transfer_8to16copy = transfer_8to16copy_altivec_c;
+          transfer_16to8copy = transfer_16to8copy_altivec_c;
+          transfer_8to16sub = transfer_8to16sub_altivec_c;
+          transfer_8to16subro = transfer_8to16subro_altivec_c;
+          transfer_8to16sub2 = transfer_8to16sub2_altivec_c;
+          transfer_16to8add = transfer_16to8add_altivec_c;
+          transfer8x8_copy = transfer8x8_copy_altivec_c;
+           
+          /* Inverse DCT */
+          idct = idct_altivec_c;
+          
+          /* Interpolation */
+          interpolate8x8_halfpel_h = interpolate8x8_halfpel_h_altivec_c;
+          interpolate8x8_halfpel_v = interpolate8x8_halfpel_v_altivec_c;
+          interpolate8x8_halfpel_hv = interpolate8x8_halfpel_hv_altivec_c;
+		  
+          interpolate8x8_avg2 = interpolate8x8_avg2_altivec_c;
+          interpolate8x8_avg4 = interpolate8x8_avg4_altivec_c;
+		  
+		  interpolate8x8_halfpel_add = interpolate8x8_halfpel_add_altivec_c;
+		  interpolate8x8_halfpel_h_add = interpolate8x8_halfpel_h_add_altivec_c;
+		  interpolate8x8_halfpel_v_add = interpolate8x8_halfpel_v_add_altivec_c;
+		  interpolate8x8_halfpel_hv_add = interpolate8x8_halfpel_hv_add_altivec_c;
+          
+          /* Colorspace conversion */
+          bgra_to_yv12 = bgra_to_yv12_altivec_c;
+          abgr_to_yv12 = abgr_to_yv12_altivec_c;
+          rgba_to_yv12 = rgba_to_yv12_altivec_c;
+          argb_to_yv12 = argb_to_yv12_altivec_c;
+          
+          yuyv_to_yv12 = yuyv_to_yv12_altivec_c;
+          uyvy_to_yv12 = uyvy_to_yv12_altivec_c;
+          
+          yv12_to_yuyv = yv12_to_yuyv_altivec_c;
+          yv12_to_uyvy = yv12_to_uyvy_altivec_c;
+          
+          /* Quantization */
+          quant_h263_intra = quant_h263_intra_altivec_c;
+          quant_h263_inter = quant_h263_inter_altivec_c;
+          dequant_h263_intra = dequant_h263_intra_altivec_c;
+          dequant_h263_inter = dequant_h263_inter_altivec_c;
 
-	if ((cpu_flags & XVID_CPU_ALTIVEC))
-	{
-		calc_cbp = calc_cbp_altivec;
-		fdct = fdct_altivec;
-		idct = idct_altivec;
-		sadInit = sadInit_altivec;
-		sad16 = sad16_altivec;
-		sad8 = sad8_altivec;
-		dev16 = dev16_altivec;
+		  dequant_mpeg_intra = dequant_mpeg_intra_altivec_c;
+		  dequant_mpeg_inter = dequant_mpeg_inter_altivec_c;
+		  
+		  /* Qpel stuff */
+		  xvid_QP_Funcs = &xvid_QP_Funcs_Altivec_C;
+		  xvid_QP_Add_Funcs = &xvid_QP_Add_Funcs_Altivec_C;
+        }
+#endif
+
+#if defined(ARCH_IS_X86_64)
+	/* For now, only XVID_CPU_ASM is looked for, so user can still
+	 * disable asm usage the usual way. When Intel EMT64 cpus will
+	 * be out, maybe we'll have to check more precisely what cpu
+	 * features there really are. */
+	if (cpu_flags & XVID_CPU_ASM) {
+		/* SIMD state flusher */
+		emms = emms_mmx;
+
+		/* DCT operators */
+		fdct = fdct_skal_x86_64;
+		idct = idct_x86_64;
+
+		/* SAD operators */
+		sad16      = sad16_x86_64;
+		sad8       = sad8_x86_64;
+		sad16bi    = sad16bi_x86_64;
+		sad8bi     = sad8bi_x86_64;
+		dev16      = dev16_x86_64;
+		sad16v	   = sad16v_x86_64;
+		sse8_16bit = sse8_16bit_x86_64;
+		sse8_8bit  = sse8_8bit_x86_64;
+
+		/* Interpolation operators */
+		interpolate8x8_halfpel_h  = interpolate8x8_halfpel_h_x86_64;
+		interpolate8x8_halfpel_v  = interpolate8x8_halfpel_v_x86_64;
+		interpolate8x8_halfpel_hv = interpolate8x8_halfpel_hv_x86_64;
+
+		interpolate8x8_halfpel_add = interpolate8x8_halfpel_add_x86_64;
+		interpolate8x8_halfpel_h_add = interpolate8x8_halfpel_h_add_x86_64;
+		interpolate8x8_halfpel_v_add = interpolate8x8_halfpel_v_add_x86_64;
+		interpolate8x8_halfpel_hv_add = interpolate8x8_halfpel_hv_add_x86_64;
+
+		interpolate8x8_6tap_lowpass_h = interpolate8x8_6tap_lowpass_h_x86_64;
+		interpolate8x8_6tap_lowpass_v = interpolate8x8_6tap_lowpass_v_x86_64;
+
+		interpolate8x8_avg2 = interpolate8x8_avg2_x86_64;
+		interpolate8x8_avg4 = interpolate8x8_avg4_x86_64;
+
+		/* Quantization related functions */
+		quant_h263_intra   = quant_h263_intra_x86_64;
+		quant_h263_inter   = quant_h263_inter_x86_64;
+		dequant_h263_intra = dequant_h263_intra_x86_64;
+		dequant_h263_inter = dequant_h263_inter_x86_64;
+		quant_mpeg_intra   = quant_mpeg_intra_x86_64;
+		quant_mpeg_inter   = quant_mpeg_inter_x86_64;
+		dequant_mpeg_intra   = dequant_mpeg_intra_x86_64;
+		dequant_mpeg_inter   = dequant_mpeg_inter_x86_64;
+
+		/* Block related functions */
+		transfer_8to16copy  = transfer_8to16copy_x86_64;
+		transfer_16to8copy  = transfer_16to8copy_x86_64;
+		transfer_8to16sub   = transfer_8to16sub_x86_64;
+		transfer_8to16subro = transfer_8to16subro_x86_64;
+		transfer_8to16sub2  = transfer_8to16sub2_x86_64;
+		transfer_8to16sub2ro= transfer_8to16sub2ro_x86_64;
+		transfer_16to8add   = transfer_16to8add_x86_64;
+		transfer8x8_copy    = transfer8x8_copy_x86_64;
+
+		/* Qpel stuff */
+		xvid_QP_Funcs = &xvid_QP_Funcs_x86_64;
+		xvid_QP_Add_Funcs = &xvid_QP_Add_Funcs_x86_64;
+
+		/* Interlacing Functions */
+		MBFieldTest = MBFieldTest_x86_64;
 	}
 #endif
 
@@ -556,7 +701,7 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
     xvid_debug = init->debug;
 #endif
 
-    return 0;
+    return(0);
 }
 
 
@@ -567,7 +712,7 @@ xvid_gbl_info(xvid_gbl_info_t * info)
 		return XVID_ERR_VERSION;
 
 	info->actual_version = XVID_VERSION;
-	info->build = "xvid-1.0.2";
+	info->build = "xvid-1.1.2";
 	info->cpu_flags = detect_cpu_flags();
 
 #if defined(_SMP) && defined(WIN32)
