@@ -423,6 +423,8 @@ LRESULT compress_begin(CODEC * codec, BITMAPINFO * lpbiInput, BITMAPINFO * lpbiO
 	xvid_plugin_2pass2_t pass2;
 	int i;
 	HANDLE hFile;
+  const quality_t* quality_preset = (codec->config.quality==quality_table_num) ?
+    &codec->config.quality_user : &quality_table[codec->config.quality];
 
 	CONFIG tmpCfg; /* if we want to alter config to suit our needs, it shouldn't be visible to user later */
 	memcpy(&tmpCfg, &codec->config, sizeof(CONFIG));
@@ -502,6 +504,15 @@ LRESULT compress_begin(CODEC * codec, BITMAPINFO * lpbiInput, BITMAPINFO * lpbiO
 		pass2.kfthreshold = codec->config.kfthreshold;
 		pass2.container_frame_overhead = 24;	/* AVI */
 
+		/* VBV */
+		pass2.vbv_size = profiles[codec->config.profile].max_vbv_size;
+		pass2.vbv_initial = (profiles[codec->config.profile].max_vbv_size*3)/4; /* 75% */
+		pass2.vbv_maxrate = profiles[codec->config.profile].max_bitrate;
+
+    // XXX: xvidcore current provides a "peak bits over 3secs" constraint.
+    //      according to the latest dxn literature, a 1sec constraint is now used
+    pass2.vbv_peakrate = profiles[codec->config.profile].vbv_peakrate * 3;
+
 		plugins[create.num_plugins].func = codec->xvid_plugin_2pass2_func;
 		plugins[create.num_plugins].param = &pass2;
 		create.num_plugins++;
@@ -547,29 +558,45 @@ LRESULT compress_begin(CODEC * codec, BITMAPINFO * lpbiInput, BITMAPINFO * lpbiO
 	create.fincr = codec->fincr;
 	create.fbase = codec->fbase;
 
-	create.max_key_interval = codec->config.max_key_interval;
+	create.max_key_interval = quality_preset->max_key_interval;
 
-	create.min_quant[0] = codec->config.min_iquant;
-	create.max_quant[0] = codec->config.max_iquant;
-	create.min_quant[1] = codec->config.min_pquant;
-	create.max_quant[1] = codec->config.max_pquant;
-	create.min_quant[2] = codec->config.min_bquant;
-	create.max_quant[2] = codec->config.max_bquant;
+	create.min_quant[0] = quality_preset->min_iquant;
+	create.max_quant[0] = quality_preset->max_iquant;
+	create.min_quant[1] = quality_preset->min_pquant;
+	create.max_quant[1] = quality_preset->max_pquant;
+	create.min_quant[2] = quality_preset->min_bquant;
+	create.max_quant[2] = quality_preset->max_bquant;
 
 	if ((profiles[codec->config.profile].flags & PROFILE_BVOP) && codec->config.use_bvop) {
-		create.max_bframes = codec->config.max_bframes;
-		create.bquant_ratio = codec->config.bquant_ratio;
-		create.bquant_offset = codec->config.bquant_offset;
 
-		if (codec->config.packed) 
-			create.global |= XVID_GLOBAL_PACKED;
+    /* dxn: prevent bframes usage if interlacing is selected */
+    if (!((profiles[codec->config.profile].flags & PROFILE_EXTRA) && codec->config.interlacing)) {
+      create.max_bframes = codec->config.max_bframes;
+		  create.bquant_ratio = codec->config.bquant_ratio;
+		  create.bquant_offset = codec->config.bquant_offset;
 
-		if (codec->config.closed_gov) 
-			create.global |= XVID_GLOBAL_CLOSED_GOP;
+		  if (codec->config.packed) 
+			  create.global |= XVID_GLOBAL_PACKED;
 
+		  create.global |= XVID_GLOBAL_CLOSED_GOP;
+
+      /* restrict max bframes */
+      if ((create.max_bframes > profiles[codec->config.profile].xvid_max_bframes) && (profiles[codec->config.profile].xvid_max_bframes >= 0))
+        create.max_bframes = profiles[codec->config.profile].xvid_max_bframes;
+
+      /* dxn: enable packed bframes */
+      if ((profiles[codec->config.profile].flags & PROFILE_PACKED)) {
+
+        create.global |= XVID_GLOBAL_PACKED;
+      }
+    }
 	}
 
-	create.frame_drop_ratio = codec->config.frame_drop_ratio;
+  /* dxn: always write divx5 userdata */
+  if ((profiles[codec->config.profile].flags & PROFILE_EXTRA))
+    create.global |= XVID_GLOBAL_DIVX5_USERDATA;
+
+	create.frame_drop_ratio = quality_preset->frame_drop_ratio;
 
 	create.num_threads = codec->config.num_threads;
 
@@ -604,6 +631,9 @@ LRESULT compress_begin(CODEC * codec, BITMAPINFO * lpbiInput, BITMAPINFO * lpbiO
 
 LRESULT compress_end(CODEC * codec)
 {
+    if (codec==NULL)
+      return ICERR_OK;
+
 	if (codec->m_hdll != NULL) {
 		if (codec->ehandle != NULL) {
 			codec->xvid_encore_func(codec->ehandle, XVID_ENC_DESTROY, NULL, NULL);
@@ -637,11 +667,17 @@ static void apply_zone_modifiers(xvid_enc_frame_t * frame, CONFIG * config, int
 		frame->vop_flags |= XVID_VOP_CHROMAOPT;
 	}
 
+	if (config->zones[i].cartoon_mode) {
+		frame->vop_flags |= XVID_VOP_CARTOON;
+		frame->motion |= XVID_ME_DETECT_STATIC_MOTION;
+	}
+
 	if ((profiles[config->profile].flags & PROFILE_BVOP) && config->use_bvop) {
 		frame->bframe_threshold = config->zones[i].bvop_threshold;
 	}
 }
 
+
 #define CALC_BI_STRIDE(width,bitcount)  ((((width * bitcount) + 31) & ~31) >> 3)
 
 LRESULT compress(CODEC * codec, ICCOMPRESS * icc)
@@ -651,6 +687,8 @@ LRESULT compress(CODEC * codec, ICCOMPRESS * icc)
 	xvid_enc_frame_t frame;
 	xvid_enc_stats_t stats;
 	int length;
+  const quality_t* quality_preset = (codec->config.quality==quality_table_num) ?
+    &codec->config.quality_user : &quality_table[codec->config.quality];
 	
 	memset(&frame, 0, sizeof(frame));
 	frame.version = XVID_VERSION;
@@ -673,12 +711,6 @@ LRESULT compress(CODEC * codec, ICCOMPRESS * icc)
 		}
 	}
 
-	if ((profiles[codec->config.profile].flags & PROFILE_REDUCED) &&
-		codec->config.reduced_resolution) {
-		frame.vol_flags |= XVID_VOL_REDUCED_ENABLE;
-		frame.vop_flags |= XVID_VOP_REDUCED;	/* XXX: need auto decion mode */
-	}
-
 	if ((profiles[codec->config.profile].flags & PROFILE_QPEL) && codec->config.qpel) {
 		frame.vol_flags |= XVID_VOL_QUARTERPEL;
 		frame.motion |= XVID_ME_QUARTERPELREFINE16 | XVID_ME_QUARTERPELREFINE8;
@@ -692,7 +724,10 @@ LRESULT compress(CODEC * codec, ICCOMPRESS * icc)
 	if ((profiles[codec->config.profile].flags & PROFILE_INTERLACE) && codec->config.interlacing)
 		frame.vol_flags |= XVID_VOL_INTERLACING;
 
-	if (codec->config.ar_mode == 0) { /* PAR */
+  /* dxn: force 1:1 picture aspect ration */
+  if ((profiles[codec->config.profile].flags & PROFILE_EXTRA)) {
+    frame.par = XVID_PAR_11_VGA;
+  } else if (codec->config.ar_mode == 0) { /* PAR */
 		if (codec->config.display_aspect_ratio != 5) {
 			frame.par = codec->config.display_aspect_ratio + 1;
 		} else {
@@ -712,32 +747,36 @@ LRESULT compress(CODEC * codec, ICCOMPRESS * icc)
 	frame.vop_flags |= XVID_VOP_HALFPEL;
 	frame.vop_flags |= XVID_VOP_HQACPRED;
 
+	if (codec->config.interlacing && codec->config.tff)
+		frame.vop_flags |= XVID_VOP_TOPFIELDFIRST;
+
+
 	if (codec->config.vop_debug) 
 		frame.vop_flags |= XVID_VOP_DEBUG;
 
-	if (codec->config.trellis_quant) {
+	if (quality_preset->trellis_quant) {
 		frame.vop_flags |= XVID_VOP_TRELLISQUANT;
 	}
 
-	if (codec->config.motion_search > 4)
-		frame.vop_flags |= XVID_VOP_INTER4V;
+  if ((profiles[codec->config.profile].flags & PROFILE_4MV)) {
+	  if (quality_preset->motion_search > 4)
+		  frame.vop_flags |= XVID_VOP_INTER4V;
+  }
 
-	if (codec->config.chromame)
+	if (quality_preset->chromame)
 		frame.motion |= XVID_ME_CHROMA_PVOP + XVID_ME_CHROMA_BVOP;
 
-	if (codec->config.cartoon_mode) {
-		frame.vop_flags |= XVID_VOP_CARTOON;
-		frame.motion |= XVID_ME_DETECT_STATIC_MOTION;
-	}
-
-	if (codec->config.turbo)
+	if (quality_preset->turbo)
  		frame.motion |= XVID_ME_FASTREFINE16 | XVID_ME_FASTREFINE8 | 
 						XVID_ME_SKIP_DELTASEARCH | XVID_ME_FAST_MODEINTERPOLATE | 
 						XVID_ME_BFRAME_EARLYSTOP;
 
-	frame.motion |= pmvfast_presets[codec->config.motion_search];
+	frame.motion |= pmvfast_presets[quality_preset->motion_search];
 
-	switch (codec->config.vhq_mode)
+	if (quality_preset->vhq_bframe) frame.vop_flags |= XVID_VOP_RD_BVOP;
+
+
+	switch (quality_preset->vhq_mode)
 	{
 	case VHQ_MODE_DECISION :
 		frame.vop_flags |= XVID_VOP_MODEDECISION_RD;
@@ -795,7 +834,7 @@ LRESULT compress(CODEC * codec, ICCOMPRESS * icc)
 	}
 
 	// force keyframe spacing in 2-pass 1st pass
-	if (codec->config.motion_search == 0)
+	if (quality_preset->motion_search == 0)
 		frame.type = XVID_TYPE_IVOP;
 
 	/* frame-based stuff */
@@ -926,7 +965,8 @@ LRESULT decompress_get_format(CODEC * codec, BITMAPINFO * lpbiInput, BITMAPINFO
 	outhdr->biPlanes = 1;
 	outhdr->biBitCount = 24;
 	outhdr->biCompression = BI_RGB;	/* sonic foundry vegas video v3 only supports BI_RGB */
-	outhdr->biSizeImage = outhdr->biWidth * CALC_BI_STRIDE(outhdr->biHeight, outhdr->biBitCount);
+	outhdr->biSizeImage = outhdr->biHeight * CALC_BI_STRIDE(outhdr->biWidth, outhdr->biBitCount);
+
 	outhdr->biXPelsPerMeter = 0;
 	outhdr->biYPelsPerMeter = 0;
 	outhdr->biClrUsed = 0;
@@ -954,6 +994,7 @@ LRESULT decompress_begin(CODEC * codec, BITMAPINFO * lpbiInput, BITMAPINFO * lpb
 	memset(&init, 0, sizeof(init));
 	init.version = XVID_VERSION;
 	init.cpu_flags = codec->config.cpu;
+  init.debug = codec->config.debug;
 	codec->xvid_global_func(0, XVID_GBL_INIT, &init, NULL);
 
 	memset(&create, 0, sizeof(create));
@@ -980,9 +1021,11 @@ LRESULT decompress_begin(CODEC * codec, BITMAPINFO * lpbiInput, BITMAPINFO * lpb
 
 	RegOpenKeyEx(XVID_REG_KEY, XVID_REG_PARENT "\\" XVID_REG_CHILD, 0, KEY_READ, &hKey);
 
+	REG_GET_N("Brightness", pp_brightness, 0);
 	REG_GET_N("Deblock_Y",  pp_dy, 0)
 	REG_GET_N("Deblock_UV", pp_duv, 0)
-	REG_GET_N("Dering",  pp_dr, 0)
+	REG_GET_N("Dering_Y",  pp_dry, 0)
+	REG_GET_N("Dering_UV", pp_druv, 0)
 	REG_GET_N("FilmEffect", pp_fe, 0)
 
 	RegCloseKey(hKey);
@@ -1030,7 +1073,7 @@ LRESULT decompress(CODEC * codec, ICDECOMPRESS * icd)
 
 		convert.input.csp = get_colorspace(icd->lpbiInput);
 		convert.input.plane[0] = icd->lpInput;
-		convert.input.stride[0] = CALC_BI_STRIDE(icd->lpbiInput->biWidth, icd->lpbiInput->biBitCount);  
+		convert.input.stride[0] = CALC_BI_STRIDE(icd->lpbiInput->biWidth, icd->lpbiInput->biBitCount);
 		if (convert.input.csp == XVID_CSP_I420 || convert.input.csp == XVID_CSP_YV12)
 			convert.input.stride[0] = (convert.input.stride[0]*2)/3;
 
@@ -1077,9 +1120,12 @@ LRESULT decompress(CODEC * codec, ICDECOMPRESS * icd)
 
 	if (pp_dy)frame.general |= XVID_DEBLOCKY;
 	if (pp_duv) frame.general |= XVID_DEBLOCKUV;
-/*	if (pp_dr) frame.general |= XVID_DERING; */
+	if (pp_dry) frame.general |= XVID_DERINGY; 
+	if (pp_druv) frame.general |= XVID_DERINGUV; 
 	if (pp_fe) frame.general |= XVID_FILMEFFECT;
 
+	frame.brightness = pp_brightness;
+
 	switch (codec->xvid_decore_func(codec->dhandle, XVID_DEC_DECODE, &frame, NULL)) 
 	{
 	case XVID_ERR_FAIL :	
